# Copyright (C) 2009 Chris Larson <clarson@kergoth.com>
# Released under the MIT license (see COPYING.MIT for the terms)
#
# gitver.bbclass provides a GITVER variable which is a (fairly) sane version,
# for use in ${PV}, extracted from the ${S} git checkout, assuming it is one.

def git_drop_tag_prefix(version):
    import re
    if re.match("v\d", version):
        return version[1:]
    else:
        return version

# Assume that the Git repository is in ${S} unless the recipe tells us otherwise
GITVER_S ?= "${S}"

GIT_TAGADJUST = "git_drop_tag_prefix(version)"
GITSHA = "${@get_git_hash('${GITVER_S}', d)}"

BS_GITVER_USE_TAG ?= "1"

# We need to give this a value since once this is in the value of PV
# Bitbake seems to want to expand it before the anonymous python below
# which causes an annoying parse error.
BS_GITVER = "not-known-yet"

python() {
    t = d.getVar("BS_GITVER_USE_TAG", True) or "1"
    if t != "1":
        use_tags = False
    else:
        use_tags = True

    gitver = ""
    for path in d.getVar('GITVER_S', True).split():
        if gitver:
            gitver += "+"
        gitver += get_git_pv(path, d, use_tags=use_tags, tagadjust=brightsign_tagadjust)

    d.setVar("BS_GITVER", gitver)
}

def brightsign_tagadjust(version):
    import re
    return re.sub(r'^([^/]*/)*', '', version)

def libmedia_tagadjust(version):
    import re
    matches = re.match('^.*/(\d+\.\d+|\d+\.\d+\.\d+|\d+\.\d+\.\d+\.\d+)([^.]+)?$', version)
    if matches:
        return matches.group(1)
    else:
        bb.fatal("libmedia_tagadjust: unknown format: " + version)

def get_git_hash2(path, git_env, d):
    import subprocess
    rev_parse_stdout, rev_parse_stderr = bb.process.run(["git", "rev-parse", "--short=12", "HEAD"], cwd=path, env=git_env, shell=False, stderr=subprocess.PIPE)
    if rev_parse_stderr != "":
        bb.fatal("get_git_hash2 %s: %s" % (path, rev_parse_stderr))
    return rev_parse_stdout.rstrip()

def get_git_hash(path, d):
    # There's no reason to run git under pseudo - it will only slow us
    # down and perhaps doing so is causing git to erroneously believe
    # that the tree is dirty sometimes.
    git_env = os.environ.copy()
    git_env["PSEUDO_DISABLED"] = "1"
    git_env["PSEUDO_UNLOAD"] = "1"
    return get_git_hash2(path, git_env, d)

# This checks to see if there are any files in the working
# tree or the index that are different to HEAD. If they are
# then we tack +local on the end of the version to indicate
# that there are local changes. This stops us matching
# anything from the shared sstate cache. We don't make PV
# change based on the mtime of the most recently changed file
# because that would cause us to fetch the source again every
# time which can often be avoided during development. (It
# turns out that "git describe" has this functionality built
# in in via its --dirty flag so we only use this if "git
# describe" fails.)
#
# To avoid triggering bug #17631 setting BS_GITVER_NEVER_MODIFIED
# disables this behaviour.

def get_git_dirty(rootdir, git_env, d):
    import subprocess
    try:
        never_modified = d.getVar('BS_GITVER_NEVER_MODIFIED', True)
        if never_modified and never_modified == "1":
            return ""
        args = [ "(git diff --no-ext-diff --ignore-space-change --quiet && git diff --no-ext-diff --ignore-space-change --cached --quiet) || echo '+local'" ]
        maybe_dirty_stdout, maybe_dirty_stderr = bb.process.run(args, cwd=rootdir, env=git_env, shell=True, stderr=subprocess.PIPE)
        if (maybe_dirty_stderr != ""):
            bb.warn("get_git_dirty for %s %s: %s" % (d.getVar('PN'), rootdir, maybe_dirty_stderr))
        maybe_dirty = maybe_dirty_stdout.rstrip()
        if maybe_dirty != "" and maybe_dirty != "+local":
            bb.fatal("git failure in gitver_notag: " + maybe_dirty)
            maybe_dirty = "+error"
        return maybe_dirty
    except Exception as exc:
        # Don't fail in here during recipe parsing
        bb.warn("git failed: " + str(exc))
        return "+local-unknown"

def get_git_pv(path, d, use_tags=False, tagadjust=None):
    import os
    import bb.process
    import subprocess

    need_machine = d.getVar('COMPATIBLE_MACHINE', True)
    if need_machine:
        import re
        compat_machines = (d.getVar('MACHINEOVERRIDES', True) or "").split(":")
        for m in compat_machines:
            if re.match(need_machine, m):
                break
        else:
            return "0.0"

    # There's no reason to run git under pseudo - it will only slow us
    # down and perhaps doing so is causing git to erroneously believe
    # that the tree is dirty sometimes.
    git_env = os.environ.copy()
    git_env["PSEUDO_DISABLED"] = "1"
    git_env["PSEUDO_UNLOAD"] = "1"

    never_modified = d.getVar('BS_GITVER_NEVER_MODIFIED', True)
    if never_modified and never_modified == "1":
        mark_if_dirty = False
    else:
        mark_if_dirty = True

    rootdir = os.path.abspath(path)
    if not use_tags:
        try:
            args = [ "git", "log", "--pretty=%ci", "-1" ]
            date = bb.process.run(args, cwd=rootdir, env=git_env)[0].rstrip()

            args = [ "git", "rev-parse", "--short=12", "HEAD" ]
            hash = bb.process.run(args, cwd=rootdir, env=git_env)[0].rstrip()

            import re, calendar
            r = re.compile("(.*) ([\+-])([0-9][0-9])([0-9][0-9])")
            m = r.match(date)
            ts = time.strptime(m.group(1), "%Y-%m-%d %H:%M:%S")
            dstoff = int(m.group(4)) * 60 + int(m.group(3)) * 3600
            if (m.group(2) == '-'):
               dstoff = -dstoff
            tv = int(calendar.timegm(ts))
            tv -= dstoff
            ts = time.gmtime(tv)
            ver = time.strftime("%Y%m%dT%H%M%S", ts) + "+" + hash
            if mark_if_dirty:
                ver += get_git_dirty(rootdir, git_env, d)
        except Exception as exc:
            # Don't fail here during recipe parsing
            bb.warn("%s(%s): %s" % (d.getVar("PN", True), rootdir, exc))
            ver = "0.0-not-in-git"
        return ver
    
    try:
        args = [ "git", "describe", "--tags", "--first-parent", "--match", 'release/*/*']
        if mark_if_dirty:
            args.extend(["--dirty=+local"])

        ver_stdout, ver_stderr = bb.process.run(args, cwd=rootdir, env=git_env, shell=False, stderr=subprocess.PIPE)
        if ver_stderr != "":
            bb.fatal("gitver describe %s: %s" % (path, ver_stderr))
        ver = ver_stdout.rstrip()
    except Exception as exc:
        # Don't fail here during recipe parsing
        bb.warn("get_git_pv %s(%s): %s" % (d.getVar("PN", True), rootdir, exc))
        ver = "0.0-not-in-git"

    if not ver:
        try:
            ver = get_git_hash2(rootdir, git_env, d)
            if mark_if_dirty:
                ver += get_git_dirty(rootdir, git_env, d)
        except Exception as exc:
            bb.fatal(str(exc))

        if ver:
            return "0.0+%s" % ver
        else:
            return "0.0"
    else:
        if tagadjust:
            return tagadjust(ver)
        else:
            return ver

BB_DONT_CACHE = "1"
